

/***

BWCCDmo2.C

Shows a Borland dialog box with a gray background, the Bordlg_gray
style.

Demonstrates placing text in a static text control.  SetWindowText(),
a call normally used to set a window caption works well for setting
these controls.

Shows that Windows style buttons are not inconsistent with the Borland
style dialog.

Some checkboxes have also been placed in the dialog to fill it out.

***/


#include <windows.h>
#include <bwcc.h>
#include <dos.h>
#include <string.h>
#include <stdlib.h>

#include "bwccdmo2.rh"

#define BUFFER_LEN 19


HWND    main_hwnd;
char    program_name[] = "BWCC Demo 2";
FARPROC dialog_procinstance;
char   *buffer[4];


// _hInstance is the instance variable declared
// and initialized in the startup code

extern HINSTANCE _hInstance;




// dialog box procedure
// this is also used as a timer callback procedure

#pragma argsused
BOOL FAR PASCAL _export AboutDlgProc(HWND hDlg, WORD message,
                                     WORD wParam, LPARAM lParam)
  {
  int  i;
  HDC  hdc;
  BOOL show_text = FALSE;
  struct time ti;
  static int  text_id[4] = { IDT_LINE1, IDT_LINE2, IDT_LINE3, IDT_LINE4 };
  static HWND text_hwnd[4];
  static UINT dialog_timer;


  // during dialog initialization
  //
  //   set up a 1 second timer, using this function as the callback proc
  //
  //   initialize the text_hwnd array to contain the window handles
  //     for the 4 static text controls
  //
  //   initialize the buffer array to blank
  //   set the 'show_text' variable so that static text will be updated

  if (message == WM_INITDIALOG)
    {
    dialog_timer = SetTimer(NULL, 0, 1000, (TIMERPROC) AboutDlgProc);

    for (i = 0; i < 4; i++)
      {
      text_hwnd[i] = GetDlgItem(hDlg, text_id[i]);
      *buffer[i] = '\0';
      }

    show_text = TRUE;
    }

  // when a timer message is received
  //
  //   place the time in the last buffer
  //   set the 'show_text' variable so it will be displayed

  else if (message == WM_TIMER)
    {
    show_text = TRUE;
    gettime(&ti);
    wsprintf(buffer[3], "time now %2d:%02d.%02d",
             ti.ti_hour, ti.ti_min, ti.ti_sec);
    }

  // when the dialog is ended
  //   discard the timer
  //
  //   call EndDialog with a return value of zero
  //     this will not return to the DialogBox() caller
  //     until after the dialog is destroyed
  //
  //   call PostQuitMessage() to end the program

  else if (message == WM_COMMAND)
    {
    if ((wParam == IDOK) || (wParam == IDCANCEL))
      {
      KillTimer(NULL, dialog_timer);
      EndDialog(hDlg, 0);
      PostQuitMessage(0);
      return TRUE;
      }
    }

  // display the buffer text in the corresponding text controls
  // move the text down one in the array of buffers
  // return TRUE to indicate we've handled the message
  
  if (show_text)
    {
    for (i = 0; i < 4; i++)
      {
      hdc = GetDC(text_hwnd[i]);
      SetWindowText(text_hwnd[i], buffer[i]);
      ReleaseDC(text_hwnd[i], hdc);

      if (i < 3)
        strcpy(buffer[i], buffer[i + 1]);
      }

    return TRUE;  
    }

  return FALSE;
  }




LRESULT CALLBACK _export WndProc(HWND hwnd, UINT message,
                                      WPARAM wParam, LPARAM lParam)
  {
  PAINTSTRUCT ps;

  if (message == WM_PAINT)
    {
    // as we've noting to write in this window,
    //   just do a BeginPaint()/EndPaint() pair
    //   to make Windows happy
    
    BeginPaint(hwnd, &ps);
    EndPaint(hwnd, &ps);

    // detect that this is the first paint message by
    //   checking if the procinstance variable has been initialized
    //
    // create and display the dialog
    //
    // when done, free the procinstance

    if (!dialog_procinstance)
      {
      dialog_procinstance =
        MakeProcInstance((FARPROC) AboutDlgProc, _hInstance);
      DialogBox(_hInstance, MAKEINTRESOURCE(GRAY_DIALOG),
                main_hwnd, (DLGPROC) dialog_procinstance);
      FreeProcInstance(dialog_procinstance);
      }

    return 0;
    }

  return DefWindowProc(hwnd, message, wParam, lParam);
  }




#pragma argsused
int PASCAL WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,
                   LPSTR command_line,  int nCmdShow)
  {
  MSG      msg;
  char    *cptr;
  int      i;
  WNDCLASS window_class;

  // allocate memory for buffers
  // complain and quit if we can't allocate
  // apportion it out among the buffer pointers

  buffer[0] = malloc(4 * BUFFER_LEN);

  if (!buffer[0])
    {
    MessageBox(GetActiveWindow(), "BWCCDemo2",
               "Allocation Failure", MB_ICONSTOP);
    return 0;
    }

  for (i = 1, cptr = buffer[0]; i < 4; i++)
    {
    cptr += BUFFER_LEN;
    buffer[i] = cptr;
    }

  if (!hPrevInstance)
    {
    window_class.style = CS_HREDRAW | CS_VREDRAW;
    window_class.lpfnWndProc = WndProc;
    window_class.cbClsExtra = window_class.cbWndExtra = 0;
    window_class.hInstance = _hInstance;
    window_class.hIcon = LoadIcon(_hInstance, "ICON_2");
    window_class.hCursor = LoadCursor(NULL, IDC_ARROW);
    window_class.hbrBackground = GetStockObject(WHITE_BRUSH);
    window_class.lpszMenuName = NULL;
    window_class.lpszClassName = program_name;
    RegisterClass(&window_class);
    BWCCGetVersion(); // make sure BWCC.DLL is loaded
    }
    
  main_hwnd = CreateWindow
    (
    program_name, program_name,
    WS_OVERLAPPEDWINDOW | WS_CLIPCHILDREN,
    CW_USEDEFAULT, CW_USEDEFAULT,
    CW_USEDEFAULT, CW_USEDEFAULT,
    NULL, NULL, _hInstance, NULL);

  ShowWindow(main_hwnd, nCmdShow);
  UpdateWindow(main_hwnd);

  while (GetMessage(&msg, NULL, 0, 0))
    {
    TranslateMessage(&msg);
    DispatchMessage(&msg);
    }

  free(buffer[0]);
  return msg.wParam;
  }



